@@ -1,20 +1,26 @@
-package KinoSearch::Analysis::LCNormalizer;
 use strict;
 use warnings;
+
+package KinoSearch::Analysis::LCNormalizer;
 use KinoSearch::Util::ToolSet;
 use base qw( KinoSearch::Analysis::Analyzer );
+use locale;
 
-our %instance_vars = __PACKAGE__->init_instance_vars();
+BEGIN { __PACKAGE__->init_instance_vars(); }
+
+use KinoSearch::Analysis::Token;
+use KinoSearch::Analysis::TokenBatch;
 
 sub analyze {
-    my ( $self, $token_batch ) = @_;
+    my ( $self, $batch ) = @_;
 
     # lc all of the terms, one by one
-    while ( $token_batch->next ) {
-        $token_batch->set_text( lc( $token_batch->get_text ) );
+    while ( my $token = $batch->next ) {
+        $token->set_text( lc( $token->get_text ) );
     }
 
-    return $token_batch;
+    $batch->reset;
+    return $batch;
 }
 
 1;
@@ -23,7 +29,7 @@ __END__
 
 =head1 NAME
 
-KinoSearch::Analysis::LCNormalizer - convert input to lower case
+KinoSearch::Analysis::LCNormalizer - Convert input to lower case.
 
 =head1 SYNOPSIS
 
@@ -47,11 +53,11 @@ though it's a no-op for now.
 
 =head1 COPYRIGHT
 
-Copyright 2005-2006 Marvin Humphrey
+Copyright 2005-2007 Marvin Humphrey
 
 =head1 LICENSE, DISCLAIMER, BUGS, etc.
 
-See L<KinoSearch|KinoSearch> version 0.10.
+See L<KinoSearch> version 0.20_01.
 
 =cut
 
