@@ -1,16 +1,22 @@
-package KinoSearch::Analysis::Stemmer;
 use strict;
 use warnings;
+
+package KinoSearch::Analysis::Stemmer;
 use KinoSearch::Util::ToolSet;
-use base qw( KinoSearch::Analysis::Analyzer Exporter );
+use base qw( KinoSearch::Analysis::Analyzer );
+
+our %supported_languages;
+
+BEGIN {
+    __PACKAGE__->init_instance_vars(
+        # constructor params / members
+        stemmifier => undef,
+    );
+}
 
 use Lingua::Stem::Snowball qw( stemmers );
-our @EXPORT_OK = qw( %supported_languages );
-
-our %instance_vars = __PACKAGE__->init_instance_vars( stemmifier => undef, );
 
 # build a list of supported languages.
-my %supported_languages;
 $supported_languages{$_} = 1 for stemmers();
 
 sub init_instance {
@@ -18,20 +24,26 @@ sub init_instance {
 
     # verify language param
     my $language = $self->{language} = lc( $self->{language} );
-    croak("Unsupported language: '$language'")
+    confess("Unsupported language: '$language'")
         unless $supported_languages{$language};
 
     # create instance of Snowball stemmer
-    $self->{stemmifier} = Lingua::Stem::Snowball->new( lang => $language );
+    $self->{stemmifier} = Lingua::Stem::Snowball->new(
+        lang     => $language,
+        encoding => 'UTF-8',
+    );
 }
 
 sub analyze {
-    my ( $self, $token_batch ) = @_;
+    my ( $self, $batch ) = @_;
 
     # replace terms with stemmed versions.
-    $self->{stemmifier}->stem_in_place( $token_batch->get_all_texts );
+    my $all_texts = $batch->get_all_texts;
+    $self->{stemmifier}->stem_in_place($all_texts);
+    $batch->set_all_texts($all_texts);
 
-    return $token_batch;
+    $batch->reset;
+    return $batch;
 }
 
 1;
@@ -40,7 +52,7 @@ __END__
 
 =head1 NAME
 
-KinoSearch::Analysis::Stemmer - reduce related words to a shared root
+KinoSearch::Analysis::Stemmer - Reduce related words to a shared root.
 
 =head1 SYNOPSIS
 
@@ -57,7 +69,7 @@ and "horsing" all become "hors" -- so that a search for 'horse' will also
 match documents containing 'horses' and 'horsing'.  
 
 This class is a wrapper around
-L<Lingua::Stem::Snowball|Lingua::Stem::Snowball>, so it supports the same
+L<Lingua::Stem::Snowball>, so it supports the same
 languages.  
 
 =head1 METHODS 
@@ -69,11 +81,11 @@ be an ISO two-letter code that Lingua::Stem::Snowball understands.
 
 =head1 COPYRIGHT
 
-Copyright 2005-2006 Marvin Humphrey
+Copyright 2005-2007 Marvin Humphrey
 
 =head1 LICENSE, DISCLAIMER, BUGS, etc.
 
-See L<KinoSearch|KinoSearch> version 0.10.
+See L<KinoSearch> version 0.20_01.
 
 =cut
 
