@@ -1,9 +1,26 @@
-package KinoSearch::QueryParser::QueryParser;
 use strict;
 use warnings;
+
+package KinoSearch::QueryParser::QueryParser;
 use KinoSearch::Util::ToolSet;
 use base qw( KinoSearch::Util::Class );
 
+BEGIN {
+    __PACKAGE__->init_instance_vars(
+        # constructor args / members
+        schema         => undef,
+        default_boolop => 'OR',
+        fields         => undef,
+        analyzer       => undef,
+        # members
+        bool_groups   => {},
+        phrases       => {},
+        bool_group_re => undef,
+        phrase_re     => undef,
+        label_inc     => 0,
+    );
+}
+
 use KinoSearch::Analysis::TokenBatch;
 use KinoSearch::Analysis::Tokenizer;
 use KinoSearch::Search::BooleanQuery;
@@ -11,23 +28,10 @@ use KinoSearch::Search::PhraseQuery;
 use KinoSearch::Search::TermQuery;
 use KinoSearch::Index::Term;
 
-our %instance_vars = __PACKAGE__->init_instance_vars(
-    # constructor args / members
-    analyzer       => undef,
-    default_boolop => 'OR',
-    default_field  => undef,
-    # members
-    bool_groups   => {},
-    phrases       => {},
-    bool_group_re => undef,
-    phrase_re     => undef,
-    label_inc     => 0,
-);
-
 sub init_instance {
     my $self = shift;
 
-    croak("default_boolop must be either 'AND' or 'OR'")
+    confess("default_boolop must be either 'AND' or 'OR'")
         unless $self->{default_boolop} =~ /^(?:AND|OR)$/;
 
     # create a random string that presumably won't appear in a search string
@@ -39,6 +43,20 @@ sub init_instance {
     # create labels which won't appear in search strings
     $self->{phrase_re}     = qr/^(_phrase$randstring\d+)/;
     $self->{bool_group_re} = qr/^(_boolgroup$randstring\d+)/;
+
+    # verify schema or analyzer
+    confess("Must supply either schema or analyzer")
+        unless ( a_isa_b( $self->{schema}, "KinoSearch::Schema" )
+        or a_isa_b( $self->{analyzer}, "KinoSearch::Analysis::Analyzer" ) );
+
+    # verify or create fields param
+    if ( !defined $self->{fields} ) {
+        my @fields = map { $_->get_name }
+            grep { $_->indexed } $self->{schema}->all_fspecs;
+        $self->{fields} = \@fields;
+    }
+    confess("Required parameter 'fields' not supplied as arrayref")
+        unless reftype( $self->{fields} ) eq 'ARRAY';
 }
 
 # regex matching a quoted string
@@ -81,7 +99,8 @@ my $field_re = qr/^
 sub parse {
     my ( $self, $qstring_orig ) = @_;
     $qstring_orig = '' unless defined $qstring_orig;
-    my $default_field  = $self->{default_field};
+    utf8::upgrade($qstring_orig);
+    my $default_fields = $self->{fields};
     my $default_boolop = $self->{default_boolop};
     my @clauses;
 
@@ -122,31 +141,13 @@ sub parse {
         }
 
         # set the field
-        my $field = s/^$field_re// ? $1 : $default_field;
+        my $fields = s/^$field_re// ? [$1] : $default_fields;
 
         # if a phrase label is detected...
         if (s/$self->{phrase_re}//) {
-            my $query;
-
             # retreive the text and analyze it
             my $orig_phrase_text = delete $self->{phrases}{$1};
-            my $token_texts      = $self->_analyze($orig_phrase_text);
-
-            # create a TermQuery, a PhraseQuery, or nothing
-            if ( @$token_texts == 1 ) {
-                my $term = KinoSearch::Index::Term->new( $field,
-                    $token_texts->[0] );
-                $query = KinoSearch::Search::TermQuery->new( term => $term );
-            }
-            elsif ( @$token_texts > 1 ) {
-                $query = KinoSearch::Search::PhraseQuery->new;
-                for my $token_text (@$token_texts) {
-                    $query->add_term(
-                        KinoSearch::Index::Term->new( $field, $token_text ),
-                    );
-                }
-            }
-
+            my $query = $self->_get_field_query( $fields, $orig_phrase_text );
             push @clauses, { query => $query, occur => $occur }
                 if defined $query;
         }
@@ -159,14 +160,9 @@ sub parse {
         }
         # what's left is probably a term
         elsif (s/([^"(\s]+)//) {
-            my $token_texts = $self->_analyze($1);
-            my @terms = map { KinoSearch::Index::Term->new( $field, $_ ) }
-                @$token_texts;
-            for my $term (@terms) {
-                my $query
-                    = KinoSearch::Search::TermQuery->new( term => $term );
-                push @clauses, { occur => $occur, query => $query };
-            }
+            my $query = $self->_get_field_query( $fields, $1 );
+            push @clauses, { occur => $occur, query => $query }
+                if defined $query;
         }
     }
 
@@ -187,18 +183,61 @@ sub parse {
     }
 }
 
-# break a string into tokens
-sub _analyze {
-    my ( $self, $string ) = @_;
+# Wrap a TermQuery/PhraseQuery to deal with multiple fields.
+sub _get_field_query {
+    my ( $self, $fields, $text ) = @_;
+    my $supplied_analyzer = $self->{analyzer};
+    my $schema            = $self->{schema};
 
-    my $token_batch = KinoSearch::Analysis::TokenBatch->new;
-    $token_batch->add_token( $string, 0, bytes::length($string) );
-    $token_batch = $self->{analyzer}->analyze($token_batch);
-    my @token_texts;
-    while ( $token_batch->next ) {
-        push @token_texts, $token_batch->get_text;
+    my @queries;
+    for my $field (@$fields) {
+        # custom analyze for each field unless override
+        my $analyzer = $supplied_analyzer;
+        $analyzer = $schema->fetch_analyzer($field) unless defined $analyzer;
+        $analyzer = $schema->fetch_analyzer()       unless defined $analyzer;
+
+        my @token_texts = grep {length} $analyzer->analyze_raw($text);
+        my $query = $self->_gen_single_field_query( $field, \@token_texts );
+        push @queries, $query if defined $query;
     }
-    return \@token_texts;
+
+    if ( @queries == 0 ) {
+        return;
+    }
+    elsif ( @queries == 1 ) {
+        return $queries[0];
+    }
+    else {
+        my $wrapper_query = KinoSearch::Search::BooleanQuery->new;
+        for my $query (@queries) {
+            $wrapper_query->add_clause(
+                query => $query,
+                occur => 'SHOULD',
+            );
+        }
+        return $wrapper_query;
+    }
+}
+
+# Create a TermQuery, a PhraseQuery, or nothing.
+sub _gen_single_field_query {
+    my ( $self, $field, $token_texts ) = @_;
+
+    if ( @$token_texts == 1 ) {
+        my $term = KinoSearch::Index::Term->new( $field, $token_texts->[0] );
+        return KinoSearch::Search::TermQuery->new( term => $term );
+    }
+    elsif ( @$token_texts > 1 ) {
+        my $phrase_query = KinoSearch::Search::PhraseQuery->new;
+        for my $token_text (@$token_texts) {
+            $phrase_query->add_term(
+                KinoSearch::Index::Term->new( $field, $token_text ),
+            );
+        }
+        return $phrase_query;
+    }
+
+    return;
 }
 
 # replace all phrases with labels
@@ -239,48 +278,86 @@ __END__
 
 =head1 NAME
 
-KinoSearch::QueryParser::QueryParser - transform a string into a Query object
+KinoSearch::QueryParser::QueryParser - Transform a string into a Query object.
 
 =head1 SYNOPSIS
 
     my $query_parser = KinoSearch::QueryParser::QueryParser->new(
-        analyzer      => $analyzer,
-        default_field => 'bodytext',
+        schema => MySchema->new,
+        fields => [ 'body' ],
     );
     my $query = $query_parser->parse( $query_string );
     my $hits  = $searcher->search( query => $query );
 
 =head1 DESCRIPTION
 
-The QueryParser accepts search strings as input and produces Query objects,
-suitable for feeding into L<KinoSearch::Searcher|KinoSearch::Searcher>.
+The QueryParser accepts UTF-8 search strings as input and produces Query
+objects, suitable for feeding into L<KinoSearch::Searcher>.
+
+=head2 Syntax
+
+The following constructs are recognized by QueryParser.
+
+=over
+
+=item *
+
+Boolean operators 'AND', 'OR', and 'AND NOT'.
+
+=item *
+
+Prepented +plus and -minus, indicating that the labeled entity should be
+either required or forbidden -- be it a single word, a phrase, or a
+parenthetical group.
+
+=item *
+
+Logical groups, delimited by parentheses.
+
+=item *
+
+Phrases, delimited by double quotes.
+
+=item *
+
+Field-specific terms, in the form of C<fieldname:termtext>.  (The field
+specified by fieldname will be used instead of the QueryParser's default
+fields).
+
+=back
 
 =head1 METHODS
 
 =head2 new
 
     my $query_parser = KinoSearch::QueryParser::QueryParser->new(
-        analyzer       => $analyzer,      # required
-        default_field  => 'bodytext',     # required
-        default_boolop => 'AND',          # default: 'OR'
+        schema         => MySchema->new,   # required
+        analyzer       => $analyzer,       # overrides schema
+        fields         => [ 'bodytext' ],  # default: indexed fields
+        default_boolop => 'AND',           # default: 'OR'
     );
 
-Constructor.  Takes hash-style parameters:
+Constructor.  Takes hash-style parameters.  Either C<searchable> or C<analyzer>
+must be supplied.
 
 =over
 
 =item *
 
-B<analyzer> - An object which subclasses
-L<KinoSearch::Analysis::Analyzer|KinoSearch::Analysis::Analyzer>.  This
-B<must> be identical to the Analyzer used at index-time, or the results won't
-match up.
+B<schema> - An object which subclasses L<KinoSearch::Schema>.
 
 =item *
 
-B<default_field> - the name of the (only) field which will be searched
-against.  If you need to search multiple fields, you need multiple QueryParser
-objects.
+B<analyzer> - An object which subclasses L<KinoSearch::Analysis::Analyzer>.
+Ordinarily, the analyzers specified by each field's definition will be used,
+but if C<analyzer> is supplied, it will override and be used for all fields.
+This can lead to mismatches between what is in the index and what is being
+searched for, so use caution.
+
+=item *
+
+B<fields> - the names of the fields which will be searched against.  By
+default, those fields which are defined as indexed in the supplied Schema.
 
 =item *
 
@@ -294,17 +371,16 @@ want only documents which match all of the query terms, set this to 'AND'.
 
     my $query = $query_parser->parse( $query_string );
 
-Turn a query string into a Query object.  Depending on the contents of the
-query string, the returned object could be any one of several subclasses of
-L<KinoSearch::Search::Query|KinoSearch::Search::Query>.
+Turn a UTF-8 query string into a Query object.  Depending on the contents of
+the query string, the returned object could be any one of several subclasses
+of L<KinoSearch::Search::Query>.
 
 =head1 COPYRIGHT
 
-Copyright 2005-2006 Marvin Humphrey
+Copyright 2005-2007 Marvin Humphrey
 
 =head1 LICENSE, DISCLAIMER, BUGS, etc.
 
-See L<KinoSearch|KinoSearch> version 0.10.
+See L<KinoSearch> version 0.20_01.
 
 =cut
-
