@@ -1,18 +1,21 @@
-package KinoSearch::Search::BooleanClause;
 use strict;
 use warnings;
+
+package KinoSearch::Search::BooleanClause;
 use KinoSearch::Util::ToolSet;
 use base qw( KinoSearch::Util::Class );
 
-our %instance_vars = __PACKAGE__->init_instance_vars(
-    occur => 'SHOULD',
-    query => undef,
-);
+BEGIN {
+    __PACKAGE__->init_instance_vars(
+        occur => 'SHOULD',
+        query => undef,
+    );
+}
 
 sub init_instance {
     my $self = shift;
 
-    croak("invalid value for 'occur': '$self->{occur}'")
+    confess("invalid value for 'occur': '$self->{occur}'")
         unless $self->{occur} =~ /^(?:MUST|MUST_NOT|SHOULD)$/;
 }
 
@@ -40,9 +43,9 @@ __END__
 
 =begin devdocs
 
-=head1 NAME
+=head1 PRIVATE CLASS
 
-KinoSearch::Search::BooleanClause - clause in a BooleanQuery
+KinoSearch::Search::BooleanClause - Clause in a BooleanQuery.
 
 =head1 DESCRIPTION 
 
@@ -50,11 +53,11 @@ A clause in a BooleanQuery.
 
 =head1 COPYRIGHT
 
-Copyright 2005-2006 Marvin Humphrey
+Copyright 2005-2007 Marvin Humphrey
 
 =head1 LICENSE, DISCLAIMER, BUGS, etc.
 
-See L<KinoSearch|KinoSearch> version 0.10.
+See L<KinoSearch> version 0.20_01.
 
 =end devdocs
 =cut
