@@ -1,21 +1,23 @@
-package KinoSearch::Search::BooleanQuery;
 use strict;
 use warnings;
+
+package KinoSearch::Search::BooleanQuery;
 use KinoSearch::Util::ToolSet;
 use base qw( KinoSearch::Search::Query );
 
+BEGIN {
+    __PACKAGE__->init_instance_vars(
+        # constructor args / members
+        disable_coord => 0,
+        # members
+        clauses          => [],
+        max_clause_count => 1024,
+    );
+    __PACKAGE__->ready_get(qw( clauses ));
+}
+
 use KinoSearch::Search::BooleanClause;
 
-our %instance_vars = __PACKAGE__->init_instance_vars(
-    # constructor args / members
-    disable_coord => 0,
-    # members
-    clauses          => [],
-    max_clause_count => 1024,
-);
-
-__PACKAGE__->ready_get(qw( clauses ));
-
 # Add an subquery tagged with boolean characteristics.
 sub add_clause {
     my $self   = shift;
@@ -72,17 +74,17 @@ sub clone {
 }
 
 package KinoSearch::Search::BooleanWeight;
-use strict;
-use warnings;
 use KinoSearch::Util::ToolSet;
 use base qw( KinoSearch::Search::Weight );
 
-use KinoSearch::Search::BooleanScorer;
+BEGIN {
+    __PACKAGE__->init_instance_vars(
+        # members
+        weights => [],
+    );
+}
 
-our %instance_vars = __PACKAGE__->init_instance_vars(
-    # members
-    weights => [],
-);
+use KinoSearch::Search::BooleanScorer;
 
 sub init_instance {
     my $self = shift;
@@ -94,6 +96,8 @@ sub init_instance {
         my $query = $clause->get_query;
         push @$weights, $query->create_weight($searcher);
     }
+
+    undef $self->{searcher};    # don't want the baggage
 }
 
 sub get_value { shift->{parent}->get_boost }
@@ -116,7 +120,7 @@ sub normalize {
 }
 
 sub scorer {
-    my ( $self, $reader ) = @_;
+    my ( $self, $ix_reader ) = @_;
 
     my $scorer = KinoSearch::Search::BooleanScorer->new(
         similarity => $self->{similarity}, );
@@ -126,7 +130,7 @@ sub scorer {
     my $i       = 0;
     for my $weight ( @{ $self->{weights} } ) {
         my $clause    = $clauses->[ $i++ ];
-        my $subscorer = $weight->scorer($reader);
+        my $subscorer = $weight->scorer($ix_reader);
         if ( defined $subscorer ) {
             $scorer->add_subscorer( $subscorer, $clause->get_occur );
         }
@@ -144,7 +148,7 @@ __END__
 
 =head1 NAME
 
-KinoSearch::Search::BooleanQuery - match boolean combinations of Queries
+KinoSearch::Search::BooleanQuery - Match boolean combinations of Queries.
 
 =head1 SYNOPSIS
 
@@ -209,7 +213,7 @@ Add a clause to the BooleanQuery.  Takes hash-style parameters:
 =item *
 
 B<query> - an object which belongs to a subclass of
-L<KinoSearch::Search::Query|KinoSearch::Search::Query>.
+L<KinoSearch::Search::Query>.
 
 =item *
 
@@ -220,11 +224,11 @@ B<occur> - must be one of three possible values: 'SHOULD', 'MUST', or
 
 =head1 COPYRIGHT
 
-Copyright 2005-2006 Marvin Humphrey
+Copyright 2005-2007 Marvin Humphrey
 
 =head1 LICENSE, DISCLAIMER, BUGS, etc.
 
-See L<KinoSearch|KinoSearch> version 0.10.
+See L<KinoSearch> version 0.20_01.
 
 =cut
 
