@@ -1,23 +1,25 @@
-package KinoSearch::Search::PhraseQuery;
 use strict;
 use warnings;
+
+package KinoSearch::Search::PhraseQuery;
 use KinoSearch::Util::ToolSet;
 use base qw( KinoSearch::Search::Query );
 
-use KinoSearch::Search::TermQuery;
-use KinoSearch::Document::Field;
-use KinoSearch::Util::ToStringUtils qw( boost_to_string );
+BEGIN {
+    __PACKAGE__->init_instance_vars(
+        # constructor args / members
+        slop => 0,
+        # members
+        field     => undef,
+        terms     => [],
+        positions => [],
+    );
+    __PACKAGE__->ready_get_set(qw( slop ));
+    __PACKAGE__->ready_get(qw( terms ));
+}
 
-our %instance_vars = __PACKAGE__->init_instance_vars(
-    # constructor args / members
-    slop => 0,
-    # members
-    field     => undef,
-    terms     => [],
-    positions => [],
-);
-__PACKAGE__->ready_get_set(qw( slop ));
-__PACKAGE__->ready_get(qw( terms ));
+use KinoSearch::Search::TermQuery;
+use KinoSearch::Util::ToStringUtils qw( boost_to_string );
 
 # Add a term/position combo to the query.  The position is specified
 # explicitly in order to allow for phrases with gaps, two terms at the same
@@ -26,7 +28,7 @@ sub add_term {
     my ( $self, $term, $position ) = @_;
     my $field = $term->get_field;
     $self->{field} = $field unless defined $self->{field};
-    croak("Mismatched fields in phrase query: '$self->{field}' '$field'")
+    confess("Mismatched fields in phrase query: '$self->{field}' '$field'")
         unless ( $field eq $self->{field} );
     if ( !defined $position ) {
         $position =
@@ -72,14 +74,12 @@ sub to_string {
 }
 
 package KinoSearch::Search::PhraseWeight;
-use strict;
-use warnings;
 use KinoSearch::Util::ToolSet;
 use base qw( KinoSearch::Search::Weight );
 
-use KinoSearch::Search::PhraseScorer;
+BEGIN { __PACKAGE__->init_instance_vars(); }
 
-our %instance_vars = __PACKAGE__->init_instance_vars();
+use KinoSearch::Search::PhraseScorer;
 
 sub init_instance {
     my $self = shift;
@@ -87,23 +87,22 @@ sub init_instance {
         = $self->{parent}->get_similarity( $self->{searcher} );
     $self->{idf} = $self->{similarity}
         ->idf( $self->{parent}->get_terms, $self->{searcher} );
+
+    undef $self->{searcher};    # don't want the baggage
 }
 
 sub scorer {
-    my ( $self, $reader ) = @_;
+    my ( $self, $ix_reader ) = @_;
     my $query = $self->{parent};
 
     # look up each term
     my @term_docs;
     for my $term ( @{ $query->{terms} } ) {
-        my $td = $reader->term_docs($term);
+        my $td = $ix_reader->term_docs($term);
 
         # bail if any one of the terms isn't in the index
         return unless defined $td;
         push @term_docs, $td;
-
-        # turn on positions
-        $td->set_read_positions(1);
     }
 
     # bail if there are no terms
@@ -113,7 +112,6 @@ sub scorer {
         weight         => $self,
         slop           => $query->{slop},
         similarity     => $self->{similarity},
-        norms_reader   => $reader->norms_reader( $query->{field} ),
         term_docs      => \@term_docs,
         phrase_offsets => $query->{positions},
     );
@@ -125,7 +123,7 @@ __END__
 
 =head1 NAME
 
-KinoSearch::Search::PhraseQuery - match ordered list of Terms
+KinoSearch::Search::PhraseQuery - Match ordered list of Terms.
 
 =head1 SYNOPSIS
 
@@ -138,8 +136,7 @@ KinoSearch::Search::PhraseQuery - match ordered list of Terms
 
 =head1 DESCRIPTION 
 
-PhraseQuery is a subclass of
-L<KinoSearch::Search::Query|KinoSearch::Search::Query> for matching against
+PhraseQuery is a subclass of L<KinoSearch::Search::Query> for matching against
 ordered collections of terms.  
 
 =head1 METHODS
@@ -155,15 +152,15 @@ Constructor.  Takes no arguments.
     $phrase_query->add_term($term);
 
 Append a term to the phrase to be matched.  Takes one argument, a
-L<KinoSearch::Index::Term|KinoSearch::Index::Term> object.
+L<KinoSearch::Index::Term> object.
 
 =head1 COPYRIGHT
 
-Copyright 2005-2006 Marvin Humphrey
+Copyright 2005-2007 Marvin Humphrey
 
 =head1 LICENSE, DISCLAIMER, BUGS, etc.
 
-See L<KinoSearch|KinoSearch> version 0.10.
+See L<KinoSearch> version 0.20_01.
 
 =cut
 
