@@ -1,14 +1,17 @@
-package KinoSearch::Search::Searchable;
 use strict;
 use warnings;
+
+package KinoSearch::Search::Searchable;
 use KinoSearch::Util::ToolSet;
 use base qw( KinoSearch::Util::Class );
 
-use KinoSearch::Search::Similarity;
-
-our %instance_vars = __PACKAGE__->init_instance_vars( similarity => undef );
-
-__PACKAGE__->ready_get_set(qw( similarity ));
+BEGIN {
+    __PACKAGE__->init_instance_vars(
+        # members
+        schema => undef,
+    );
+    __PACKAGE__->ready_get(qw( schema ));
+}
 
 =begin comment
 
@@ -52,13 +55,24 @@ sub max_doc { shift->abstract_death }
 
     my $doc =  $searchable->fetch_doc($doc_num);
 
-Generate a Doc object, retrieving the stored fields from the invindex.
+Retrieve stored fields as a hashref.
 
 =end comment
 =cut
 
 sub fetch_doc { shift->abstract_death }
 
+=begin comment
+
+    my $doc_vector =  $searchable->fetch_doc_vec($doc_num);
+
+Generate a DocVector object from the relevant term vector files.
+
+=end comment
+=cut
+
+sub fetch_doc_vec { shift->abstract_death }
+
 =begin comment
 
     my $doc_freq = $searchable->doc_freq($term);
@@ -71,12 +85,34 @@ Weights.
 
 sub doc_freq { shift->abstract_death }
 
-# not sure these are needed (call $query->create_weight($searcher) instead)
-sub create_weight { shift->unimplemented_death }
-sub rewrite_query { shift->unimplemented_death }
+=begin comment
 
-# needed by MultiSearcher?
-sub doc_freqs { shift->unimplemented_death }
+    my $sim       = $searchable->get_similarity;
+    my $field_sim = $searchable->get_similarity($field_name);
+
+Retrieve a Similarity object.  If a field name is included, retrieve the
+Similarity instance for that field only.
+
+=end comment
+=cut
+
+sub get_similarity {
+    my ( $self, $field_name ) = @_;
+
+    # prevent undef warning -- we always want to return a similarity
+    $field_name ||= "";
+
+    return $self->{schema}->fetch_sim($field_name);
+}
+
+# Factory method for turning a Query into a Weight.
+sub create_weight { shift->abstract_death }
+
+sub doc_freqs {
+    my ( $self, $terms ) = @_;
+    my @doc_freqs = map { $self->doc_freq($_) } @$terms;
+    return \@doc_freqs;
+}
 
 sub close { }
 
@@ -86,22 +122,22 @@ __END__
 
 =begin devdocs
 
-=head1 NAME
+=head1 PRIVATE CLASS
 
-KinoSearch::Search::Searchable - base class for searching an invindex
+KinoSearch::Search::Searchable - Base class for searching an InvIndex.
 
 =head1 DESCRIPTION 
 
-Abstract base class for objects which search an invindex.  The most prominent
+Abstract base class for objects which search an InvIndex.  The most prominent
 subclass is KinoSearch::Searcher.
 
 =head1 COPYRIGHT
 
-Copyright 2005-2006 Marvin Humphrey
+Copyright 2005-2007 Marvin Humphrey
 
 =head1 LICENSE, DISCLAIMER, BUGS, etc.
 
-See L<KinoSearch|KinoSearch> version 0.10.
+See L<KinoSearch> version 0.20_01.
 
 =end devdocs
 =cut
