@@ -1,12 +1,19 @@
-package KinoSearch::Search::TermQuery;
 use strict;
 use warnings;
+
+package KinoSearch::Search::TermQuery;
 use KinoSearch::Util::ToolSet;
 use base qw( KinoSearch::Search::Query );
 
 use KinoSearch::Util::ToStringUtils qw( boost_to_string );
 
-our %instance_vars = __PACKAGE__->init_instance_vars( term => undef );
+BEGIN {
+    __PACKAGE__->init_instance_vars(
+        # constructor params / members
+        term => undef,
+    );
+    __PACKAGE__->ready_get(qw( term ));
+}
 
 sub init_instance {
     my $self = shift;
@@ -14,8 +21,6 @@ sub init_instance {
         unless a_isa_b( $self->{term}, 'KinoSearch::Index::Term' );
 }
 
-sub get_term { shift->{term} }
-
 sub create_weight {
     my ( $self, $searcher ) = @_;
     my $weight = KinoSearch::Search::TermWeight->new(
@@ -35,11 +40,15 @@ sub to_string {
 
 }
 
+sub get_similarity {
+    my ( $self, $searcher ) = @_;
+    my $field_name = $self->{term}->get_field;
+    return $searcher->get_similarity($field_name);
+}
+
 sub equals { shift->todo_death }
 
 package KinoSearch::Search::TermWeight;
-use strict;
-use warnings;
 use KinoSearch::Util::ToolSet;
 use base qw( KinoSearch::Search::Weight );
 
@@ -55,20 +64,22 @@ sub init_instance {
 
     $self->{idf} = $self->{similarity}
         ->idf( $self->{parent}->get_term, $self->{searcher} );
+
+    # kill this because we don't want its baggage.
+    undef $self->{searcher};
 }
 
 sub scorer {
-    my ( $self, $reader ) = @_;
+    my ( $self, $ix_reader ) = @_;
     my $term      = $self->{parent}{term};
-    my $term_docs = $reader->term_docs($term);
+    my $term_docs = $ix_reader->term_docs($term);
     return unless defined $term_docs;
+    return unless $term_docs->get_doc_freq;
 
-    my $norms_reader = $reader->norms_reader( $term->get_field );
     return KinoSearch::Search::TermScorer->new(
-        weight       => $self,
-        term_docs    => $term_docs,
-        similarity   => $self->{similarity},
-        norms_reader => $norms_reader,
+        weight     => $self,
+        term_docs  => $term_docs,
+        similarity => $self->{similarity},
     );
 }
 
@@ -83,7 +94,7 @@ __END__
 
 =head1 NAME
 
-KinoSearch::Search::TermQuery - match individual Terms
+KinoSearch::Search::TermQuery - Match individual Terms.
 
 =head1 SYNOPSIS
 
@@ -95,10 +106,9 @@ KinoSearch::Search::TermQuery - match individual Terms
 
 =head1 DESCRIPTION 
 
-TermQuery is a subclass of
-L<KinoSearch::Search::Query|KinoSearch::Search::Query> for matching individual
-L<Terms|KinoSearch::Index::Term>.  Note that since Term objects are associated
-with one and only one field, so are TermQueries.
+TermQuery is a subclass of L<KinoSearch::Search::Query> for matching
+individual L<Terms|KinoSearch::Index::Term>.  Note that since Term objects are
+associated with one and only one field, so are TermQueries.
 
 =head1 METHODS
 
@@ -120,11 +130,11 @@ B<term> - a L<KinoSearch::Index::Term>.
 
 =head1 COPYRIGHT
 
-Copyright 2005-2006 Marvin Humphrey
+Copyright 2005-2007 Marvin Humphrey
 
 =head1 LICENSE, DISCLAIMER, BUGS, etc.
 
-See L<KinoSearch|KinoSearch> version 0.10.
+See L<KinoSearch> version 0.20_01.
 
 =cut
 
