@@ -1,20 +1,23 @@
-package KinoSearch::Search::Weight;
 use strict;
 use warnings;
+
+package KinoSearch::Search::Weight;
 use KinoSearch::Util::ToolSet;
 use base qw( KinoSearch::Util::Class );
 
-our %instance_vars = __PACKAGE__->init_instance_vars(
-    # constructor args / members
-    parent   => undef,
-    searcher => undef,
-    # members
-    similarity   => undef,
-    value        => 0,
-    idf          => undef,
-    query_norm   => undef,
-    query_weight => undef,
-);
+BEGIN {
+    __PACKAGE__->init_instance_vars(
+        # constructor args / members
+        parent   => undef,
+        searcher => undef,
+        # members
+        similarity   => undef,
+        value        => 0,
+        idf          => undef,
+        query_norm   => undef,
+        query_weight => undef,
+    );
+}
 
 # Return the Query that the Weight was derived from.
 sub get_query { shift->{parent} }
@@ -70,9 +73,9 @@ sub to_string {
 
 =begin devdocs
 
-=head1 NAME
+=head1 PRIVATE CLASS
 
-KinoSearch::Search::Weight - Searcher-dependent transformation of a Query
+KinoSearch::Search::Weight - Searcher-dependent transformation of a Query.
 
 =head1 SYNOPSIS
 
@@ -89,11 +92,11 @@ are encapsulated within a class.
 
 =head1 COPYRIGHT
 
-Copyright 2005-2006 Marvin Humphrey
+Copyright 2005-2007 Marvin Humphrey
 
 =head1 LICENSE, DISCLAIMER, BUGS, etc.
 
-See L<KinoSearch|KinoSearch> version 0.10.
+See L<KinoSearch> version 0.20_01.
 
 =end devdocs
 =cut
