@@ -1,6 +1,7 @@
-package KinoSearch::Util::ToolSet;
 use strict;
 use warnings;
+
+package KinoSearch::Util::ToolSet;
 use bytes;
 no bytes;
 
@@ -21,8 +22,8 @@ use Scalar::Util qw(
     looks_like_number
 );
 use KinoSearch qw( K_DEBUG kdump );
-use KinoSearch::Util::VerifyArgs qw( verify_args a_isa_b );
-use KinoSearch::Util::MathUtils qw( ceil );
+use KinoSearch::Util::VerifyArgs qw( verify_args kerror a_isa_b );
+use File::Spec::Functions qw();
 
 our @EXPORT = qw(
     carp
@@ -46,8 +47,7 @@ our @EXPORT = qw(
 
     verify_args
     a_isa_b
-
-    ceil
+    kerror
 );
 
 1;
@@ -56,7 +56,7 @@ __END__
 
 =head1 NAME
 
-KinoSearch::Util::ToolSet - namespace pollution
+KinoSearch::Util::ToolSet - Namespace pollution.
 
 =head1 PRIVATE CLASS
 
@@ -71,8 +71,8 @@ warning.  Do not use it on its own.
 
 KinoSearch::Util::ToolSet makes a slew of commonly needed symbols available to
 other modules in the KinoSearch suite.  At one time it was implemented using
-David Golden's L<ToolSet|ToolSet> module, but in keeping with the philosophy
-of minimizing non-core dependencies, a 90% solution based on Exporter has been
+David Golden's L<ToolSet> module, but in keeping with the philosophy of
+minimizing non-core dependencies, a 90% solution based on Exporter has been
 substituted.
 
     use KinoSearch::Util::ToolSet;
@@ -96,6 +96,7 @@ substituted.
     use KinoSearch qw( K_DEBUG kdump );
     use KinoSearch::Util::VerifyArgs qw( verify_args a_isa_b );
     use KinoSearch::Util::MathUtils qw( ceil );
+    use File::Spec::Functions qw();
 
 Two issues deserve special attention.
 
@@ -104,17 +105,16 @@ bytes:: namespace, such as bytes::length, will be available, while still
 keeping character semantics enabled by default -- so regexes work as expected,
 etc.
 
-Second, the C<use KinoSearch> line does a LOT more than it appears to at
-first glance -- it loads ALL of the XS routines in the entire KinoSearch
-suite.  See L<KinoSearch::Docs::DevGuide|KinoSearch::Docs::DevGuide> for an
-explanation.
+Second, the C<use KinoSearch> line does a LOT more than it appears to at first
+glance -- it loads ALL of the XS routines in the entire KinoSearch suite.  See
+L<KinoSearch::Docs::DevGuide> for an explanation.
 
 =head1 COPYRIGHT
 
-Copyright 2005-2006 Marvin Humphrey
+Copyright 2005-2007 Marvin Humphrey
 
 =head1 LICENSE, DISCLAIMER, BUGS, etc.
 
-See L<KinoSearch|KinoSearch> version 0.10.
+See L<KinoSearch> version 0.20_01.
 
 =cut
